package com.buzzaa.wildfire;

import java.net.SocketException;

import junit.framework.TestCase;

import org.jivesoftware.smack.BlockableXMPPConnection;
import org.jivesoftware.smack.ConnectionConfiguration;
import org.jivesoftware.smack.Roster;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.packet.Packet;
import org.jivesoftware.smack.packet.Presence;
import org.xmpp.packet.JID;

/**
 * Testcase to check if idle connections are being dropped by the server. Be
 * sure to adjust the constant values to your needs. Adjusting the idle timeout
 * treshold on your server (by settting the xmpp.client.idle property) should
 * speed up this test.
 * 
 * @author Guus der Kinderen, guus@nimbuzz.com
 */
public class IdleSessionTimeoutTest extends TestCase
{
	/**
	 * XMPP domain to test.
	 */
	private static final String DOMAIN;

	/**
	 * Specific server within DOMAIN.
	 */
	private static final String SERVER;

	/**
	 * Username of the user that's should be timing out.
	 */
	private static final String USER1 = "connectiontimeouttester1";

	/**
	 * Username of a user that is subscribed to USER1, which checks if the
	 * server actually sends 'unavailable' for user1.
	 */
	private static final String USER2 = "connectiontimeouttester2";

	/**
	 * The same password is used for both USER1 and USER2.
	 */
	private static final String PASSWORD = "test";

	/**
	 * Connection configuration to setup the Smack connection.
	 */
	private static final ConnectionConfiguration CONF = new ConnectionConfiguration(
		SERVER, 5222, DOMAIN);

	/**
	 * Connection for user1. This connection can be prohibited to sent any
	 * packets.
	 */
	private static final BlockableXMPPConnection CON1 = new BlockableXMPPConnection(
		CONF);

	/**
	 * Connection for user2.
	 */
	private static final XMPPConnection CON2 = new XMPPConnection(CONF);

	/**
	 * Bare JID representation for user1.
	 */
	private static final String USER1BAREJID = new JID(USER1, DOMAIN, null).toBareJID();

	/**
	 * Set this to the amount of seconds that Openfire takes to disconnect an
	 * idle session.
	 * 
	 * Openfire configures the timeout (in millis) in the property
	 * 'xmpp.client.idle'.
	 * 
	 * Note that this test will wait for disconnections for a period at least
	 * twice longer, because Openfire waits this period after the server has
	 * started, before activating the task to cleanup idle sessions.
	 */
	private static final int TIMEOUT_IN_SECS = 90;

	/*
	 * (non-Javadoc)
	 * 
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception
	{
		super.setUp();
		CON1.connect();
		CON1.login(USER1, PASSWORD);
		CON2.connect();
		CON2.login(USER2, PASSWORD);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see junit.framework.TestCase#tearDown()
	 */
	@Override
	protected void tearDown() throws Exception
	{
		if (CON1 != null && CON1.isConnected())
		{
			if (!CON1.isOutgoingTrafficEnabled())
			{
				CON1.enableOutgoingTraffic(true);
			}
			CON1.disconnect();
		}
		if (CON2 != null && CON2.isConnected())
		{
			CON2.disconnect();
		}
		super.tearDown();
	}

	/**
	 * Tests if the XMPP domain closes an idle connection. This test uses two
	 * connections to the domain: CON1 is the connection that is being idle and
	 * the one that should be closed by the domain. CON2 is a connection of a
	 * user that should be subscribed to the user of CON1. It's used to verify
	 * that the XMPP domain sends out 'unavailable' for the user that's being
	 * idle.
	 * 
	 * @throws InterruptedException
	 * @throws SocketException
	 */
	@SuppressWarnings("null")
	public void testTimeout() throws InterruptedException, SocketException
	{
		assertTrue(CON1.isConnected());
		assertTrue(CON2.isConnected());

		final Roster roster2 = CON2.getRoster();
		assertNotNull(roster2);

		// Wait for Smack async roster handling to see presence.
		Presence presence = null;
		int step = 50;
		for (int timeout = 5000; timeout > 0; timeout -= step)
		{
			presence = roster2.getPresence(USER1BAREJID);
			if (presence != null)
			{
				break;
			}

			Thread.sleep(step);
		}
		assertNotNull("USER2 should get presence information regarding USER1",
			presence);
		assertEquals(Presence.Type.available, presence.getType());

		CON1.enableOutgoingTraffic(false);

		// Wildfire is known to wait the timeout period before activating the
		// task that cleans up idle connections.
		int timeout = Math.round(2.5f * TIMEOUT_IN_SECS);
		System.out.println("This testcase now sleeps up to " + timeout
				+ " seconds, to see if idle connections are disconnected"
				+ " after this time. Note that any exceptions that might"
				+ " be printed to std.err are expected: They are generated"
				+ " by Smack, which is detecting that the server closes the"
				+ " socket to the idle connection.");

		step = 1000;
		for (timeout *= 1000; timeout > 0; timeout -= step)
		{
			if (roster2.getPresence(USER1BAREJID) == null
					|| !roster2.getPresence(USER1BAREJID).getType().equals(
						Presence.Type.available))
			{
				break;
			}

			// keep our verification connection up by sending whitespaces.
			CON2.sendPacket(new Packet()
			{
				@Override
				public String toXML()
				{
					return " ";
				}
			});

			Thread.sleep(step);
		}

		assertTrue(roster2.getPresence(USER1BAREJID) == null
				|| !roster2.getPresence(USER1BAREJID).getType().equals(
					Presence.Type.available));
	}
}
